/* This file was generated by syncqt. */
#ifndef QT_QTPACKETPROTOCOL_VERSION_H
#define QT_QTPACKETPROTOCOL_VERSION_H

#define QTPACKETPROTOCOL_VERSION_STR "6.11.2"

#define QTPACKETPROTOCOL_VERSION 0x0601102

#endif // QT_QTPACKETPROTOCOL_VERSION_H
