/* This file was generated by syncqt. */
#ifndef QT_QTQMLTYPEREGISTRAR_VERSION_H
#define QT_QTQMLTYPEREGISTRAR_VERSION_H

#define QTQMLTYPEREGISTRAR_VERSION_STR "6.11.2"

#define QTQMLTYPEREGISTRAR_VERSION 0x0601102

#endif // QT_QTQMLTYPEREGISTRAR_VERSION_H
