/* This file was generated by syncqt. */
#ifndef QT_QTQUICKCONTROLS2BASIC_VERSION_H
#define QT_QTQUICKCONTROLS2BASIC_VERSION_H

#define QTQUICKCONTROLS2BASIC_VERSION_STR "6.11.2"

#define QTQUICKCONTROLS2BASIC_VERSION 0x0601102

#endif // QT_QTQUICKCONTROLS2BASIC_VERSION_H
