/* This file was generated by syncqt. */
#ifndef QT_QTQUICKTESTUTILS_VERSION_H
#define QT_QTQUICKTESTUTILS_VERSION_H

#define QTQUICKTESTUTILS_VERSION_STR "6.11.2"

#define QTQUICKTESTUTILS_VERSION 0x0601102

#endif // QT_QTQUICKTESTUTILS_VERSION_H
