/* This file was generated by syncqt. */
#ifndef QT_QTQMLLOCALSTORAGE_VERSION_H
#define QT_QTQMLLOCALSTORAGE_VERSION_H

#define QTQMLLOCALSTORAGE_VERSION_STR "6.11.2"

#define QTQMLLOCALSTORAGE_VERSION 0x0601102

#endif // QT_QTQMLLOCALSTORAGE_VERSION_H
