/* This file was generated by syncqt. */
#ifndef QT_QTQUICKVECTORIMAGE_VERSION_H
#define QT_QTQUICKVECTORIMAGE_VERSION_H

#define QTQUICKVECTORIMAGE_VERSION_STR "6.11.2"

#define QTQUICKVECTORIMAGE_VERSION 0x0601102

#endif // QT_QTQUICKVECTORIMAGE_VERSION_H
